@@ -70,12 +70,13 @@ static char *tmp_path(char *path)
 static void saveDoc(char *current_path, fz_document *doc)
 {
 	char *tmp;
-	fz_write_options opts;
+	pdf_document *idoc = pdf_specifics(ctx, doc);
+	pdf_write_options opts = { 0 };
+
 	opts.do_incremental = 1;
-	opts.do_ascii = 0;
-	opts.do_expand = 0;
-	opts.do_garbage = 0;
-	opts.do_linear = 0;
+
+	if (!idoc)
+		return;
 
 	tmp = tmp_path(current_path);
 	if (tmp)
@@ -88,7 +89,8 @@ static void saveDoc(char *current_path, fz_document *doc)
 			FILE *fin = fopen(current_path, "rb");
 			FILE *fout = fopen(tmp, "wb");
 			char buf[256];
-			int n, err = 1;
+			size_t n;
+			int err = 1;
 
 			if (fin && fout)
 			{
@@ -104,7 +106,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 			if (!err)
 			{
-				fz_write_document(doc, tmp, &opts);
+				pdf_save_document(ctx, idoc, tmp, &opts);
 				written = 1;
 			}
 		}
@@ -124,6 +126,35 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 @implementation MuDocumentController
 {
+	fz_document *doc;
+	MuDocRef *docRef;
+	NSString *key;
+	char *filePath;
+	BOOL reflowMode;
+	MuOutlineController *outline;
+	UIScrollView *canvas;
+	UILabel *indicator;
+	UISlider *slider;
+	UISearchBar *searchBar;
+	UIBarButtonItem *nextButton, *prevButton, *cancelButton, *searchButton, *outlineButton, *linkButton;
+	UIBarButtonItem *moreButton;
+	UIBarButtonItem *shareButton, *printButton, *annotButton;
+	UIBarButtonItem *highlightButton, *underlineButton, *strikeoutButton;
+	UIBarButtonItem *inkButton;
+	UIBarButtonItem *tickButton;
+	UIBarButtonItem *deleteButton;
+	UIBarButtonItem *reflowButton;
+	UIBarButtonItem *backButton;
+	UIBarButtonItem *sliderWrapper;
+	int barmode;
+	int searchPage;
+	int cancelSearch;
+	int showLinks;
+	int width; // current screen size
+	int height;
+	int current; // currently visible page
+	int scroll_animating; // stop view updates during scrolling animations
+	float scale; // scale applied to views (only used in reflow mode)
 	BOOL _isRotating;
 }
 
@@ -144,7 +175,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 	dispatch_sync(queue, ^{});
 
-	fz_outline *root = fz_load_outline(doc);
+	fz_outline *root = fz_load_outline(ctx, doc);
 	if (root) {
 		NSMutableArray *titles = [[NSMutableArray alloc] init];
 		NSMutableArray *pages = [[NSMutableArray alloc] init];
@@ -153,7 +184,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 			outline = [[MuOutlineController alloc] initWithTarget: self titles: titles pages: pages];
 		[titles release];
 		[pages release];
-		fz_free_outline(ctx, root);
+		fz_drop_outline(ctx, root);
 	}
 
 	return self;
@@ -196,8 +227,8 @@ static void saveDoc(char *current_path, fz_document *doc)
 {
 	[[NSUserDefaults standardUserDefaults] setObject: key forKey: @"OpenDocumentKey"];
 
-	current = [[NSUserDefaults standardUserDefaults] integerForKey: key];
-	if (current < 0 || current >= fz_count_pages(doc))
+	current = (int)[[NSUserDefaults standardUserDefaults] integerForKey: key];
+	if (current < 0 || current >= fz_count_pages(ctx, doc))
 		current = 0;
 
 	UIView *view = [[UIView alloc] initWithFrame: CGRectZero];
@@ -241,7 +272,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 	slider = [[UISlider alloc] initWithFrame: CGRectZero];
 	[slider setMinimumValue: 0];
-	[slider setMaximumValue: fz_count_pages(doc) - 1];
+	[slider setMaximumValue: fz_count_pages(ctx, doc) - 1];
 	[slider addTarget: self action: @selector(onSlide:) forControlEvents: UIControlEventValueChanged];
 
 	if ([[[UIDevice currentDevice] systemVersion] floatValue] < 7.0)
@@ -323,6 +354,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 - (void) viewWillAppear: (BOOL)animated
 {
+	[super viewWillAppear:animated];
 	[self setTitle: [key lastPathComponent]];
 
 	[slider setValue: current];
@@ -330,7 +362,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 	if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 7.0)
 		[[[self navigationController] toolbar] addSubview:slider];
 
-	[indicator setText: [NSString stringWithFormat: @" %d of %d ", current+1, fz_count_pages(doc)]];
+	[indicator setText: [NSString stringWithFormat: @" %d of %d ", current+1, fz_count_pages(ctx, doc)]];
 
 	[[self navigationController] setToolbarHidden: NO animated: animated];
 }
@@ -344,7 +376,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 	height = size.height;
 
 	[canvas setContentInset: UIEdgeInsetsZero];
-	[canvas setContentSize: CGSizeMake(fz_count_pages(doc) * width, height)];
+	[canvas setContentSize: CGSizeMake(fz_count_pages(ctx, doc) * width, height)];
 	[canvas setContentOffset: CGPointMake(current * width, 0)];
 
 	[sliderWrapper setWidth: SLIDER_W];
@@ -360,7 +392,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 	[[[self navigationController] toolbar] setNeedsLayout]; // force layout!
 
 	// use max_width so we don't clamp the content offset too early during animation
-	[canvas setContentSize: CGSizeMake(fz_count_pages(doc) * max_width, height)];
+	[canvas setContentSize: CGSizeMake(fz_count_pages(ctx, doc) * max_width, height)];
 	[canvas setContentOffset: CGPointMake(current * width, 0)];
 
 	for (UIView<MuPageView> *view in [canvas subviews]) {
@@ -379,11 +411,13 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 - (void) viewDidAppear: (BOOL)animated
 {
+	[super viewDidAppear:animated];
 	[self scrollViewDidScroll: canvas];
 }
 
 - (void) viewWillDisappear: (BOOL)animated
 {
+	[super viewWillDisappear:animated];
 	if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 7.0)
 		[slider removeFromSuperview];
 
@@ -543,14 +577,15 @@ static void saveDoc(char *current_path, fz_document *doc)
 {
 	NSURL *url = [NSURL fileURLWithPath:[NSString stringWithUTF8String:filePath]];
 	UIActivityViewController *cont = [[UIActivityViewController alloc] initWithActivityItems:[NSArray arrayWithObject:url] applicationActivities:nil];
+	cont.popoverPresentationController.barButtonItem = shareButton;
 	[self presentViewController:cont animated:YES completion:nil];
 	[cont release];
 }
 
 - (void) onShare: (id)sender
 {
-	pdf_document *idoc = pdf_specifics(doc);
-	if (idoc && pdf_has_unsaved_changes(idoc))
+	pdf_document *idoc = pdf_specifics(ctx, doc);
+	if (idoc && pdf_has_unsaved_changes(ctx, idoc))
 	{
 		UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:AlertTitle message:ShareAlertMessage delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Save and Share", nil];
 		[alertView show];
@@ -711,8 +746,8 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 - (void) onBack: (id)sender
 {
-	pdf_document *idoc = pdf_specifics(doc);
-	if (idoc && pdf_has_unsaved_changes(idoc))
+	pdf_document *idoc = pdf_specifics(ctx, doc);
+	if (idoc && pdf_has_unsaved_changes(ctx, idoc))
 	{
 		UIAlertView *saveAlert = [[UIAlertView alloc]
 			initWithTitle:AlertTitle message:CloseAlertMessage delegate:self
@@ -794,7 +829,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 	cancelSearch = NO;
 
 	dispatch_async(queue, ^{
-		for (int i = start; i >= 0 && i < fz_count_pages(doc); i += dir) {
+		for (int i = start; i >= 0 && i < fz_count_pages(ctx, doc); i += dir) {
 			int n = search_page(doc, i, needle, NULL);
 			if (n) {
 				dispatch_async(dispatch_get_main_queue(), ^{
@@ -865,7 +900,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 {
 	int number = [slider value];
 	if ([slider isTracking])
-		[indicator setText: [NSString stringWithFormat: @" %d of %d ", number+1, fz_count_pages(doc)]];
+		[indicator setText: [NSString stringWithFormat: @" %d of %d ", number+1, fz_count_pages(ctx, doc)]];
 	else
 		[self gotoPage: number animated: NO];
 }
@@ -999,7 +1034,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 	[[NSUserDefaults standardUserDefaults] setInteger: current forKey: key];
 
-	[indicator setText: [NSString stringWithFormat: @" %d of %d ", current+1, fz_count_pages(doc)]];
+	[indicator setText: [NSString stringWithFormat: @" %d of %d ", current+1, fz_count_pages(ctx, doc)]];
 	[slider setValue: current];
 
 	// swap the distant page views out
@@ -1026,7 +1061,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 - (void) createPageView: (int)number
 {
-	if (number < 0 || number >= fz_count_pages(doc))
+	if (number < 0 || number >= fz_count_pages(ctx, doc))
 		return;
 	int found = 0;
 	for (UIView<MuPageView> *view in [canvas subviews])
@@ -1049,8 +1084,8 @@ static void saveDoc(char *current_path, fz_document *doc)
 {
 	if (number < 0)
 		number = 0;
-	if (number >= fz_count_pages(doc))
-		number = fz_count_pages(doc) - 1;
+	if (number >= fz_count_pages(ctx, doc))
+		number = fz_count_pages(ctx, doc) - 1;
 	if (current == number)
 		return;
 	if (animated) {
@@ -1073,7 +1108,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 
 		[canvas setContentOffset: CGPointMake(number * width, 0)];
 		[slider setValue: number];
-		[indicator setText: [NSString stringWithFormat: @" %d of %d ", number+1, fz_count_pages(doc)]];
+		[indicator setText: [NSString stringWithFormat: @" %d of %d ", number+1, fz_count_pages(ctx, doc)]];
 
 		[UIView commitAnimations];
 	} else {
@@ -1123,7 +1158,7 @@ static void saveDoc(char *current_path, fz_document *doc)
 	// We need to set these here, because during the animation we may use a wider
 	// size (the maximum of the landscape/portrait widths), to avoid clipping during
 	// the rotation.
-	[canvas setContentSize: CGSizeMake(fz_count_pages(doc) * width, height)];
+	[canvas setContentSize: CGSizeMake(fz_count_pages(ctx, doc) * width, height)];
 	[canvas setContentOffset: CGPointMake(current * width, 0)];
 }
 
