@@ -1,11 +1,21 @@
 #include "mupdf/fitz.h"
 
 #include <ft2build.h>
+#include "hb.h"
+#include "hb-ft.h"
+
 #include FT_FREETYPE_H
 #include FT_ADVANCES_H
 #include FT_STROKER_H
+#include FT_TRUETYPE_TABLES_H
+#include FT_TRUETYPE_TAGS_H
 
 #define MAX_BBOX_TABLE_SIZE 4096
+#define MAX_ADVANCE_CACHE 4096
+
+#ifndef FT_SFNT_OS2
+#define FT_SFNT_OS2 ft_sfnt_os2
+#endif
 
 /* 20 degrees */
 #define SHEAR 0.36397f
@@ -28,12 +38,10 @@ fz_new_font(fz_context *ctx, const char *name, int use_glyph_bbox, int glyph_cou
 
 	font->ft_face = NULL;
 	font->ft_substitute = 0;
-	font->ft_bold = 0;
-	font->ft_italic = 0;
-	font->ft_hint = 0;
-
-	font->ft_buffer = NULL;
-	font->ft_filepath = NULL;
+	font->fake_bold = 0;
+	font->fake_italic = 0;
+	font->force_hinting = 0;
+	font->has_opentype = 0;
 
 	font->t3matrix = fz_identity;
 	font->t3resources = NULL;
@@ -49,10 +57,11 @@ fz_new_font(fz_context *ctx, const char *name, int use_glyph_bbox, int glyph_cou
 	font->bbox.x1 = 1;
 	font->bbox.y1 = 1;
 
+	font->glyph_count = glyph_count;
+
 	font->use_glyph_bbox = use_glyph_bbox;
 	if (use_glyph_bbox && glyph_count <= MAX_BBOX_TABLE_SIZE)
 	{
-		font->bbox_count = glyph_count;
 		font->bbox_table = fz_malloc_array(ctx, glyph_count, sizeof(fz_rect));
 		for (i = 0; i < glyph_count; i++)
 			font->bbox_table[i] = fz_infinite_rect;
@@ -61,7 +70,6 @@ fz_new_font(fz_context *ctx, const char *name, int use_glyph_bbox, int glyph_cou
 	{
 		if (use_glyph_bbox)
 			fz_warn(ctx, "not building glyph bbox table for font '%s' with %d glyphs", font->name, glyph_count);
-		font->bbox_count = 0;
 		font->bbox_table = NULL;
 	}
 
@@ -119,14 +127,12 @@ fz_drop_font(fz_context *ctx, fz_font *font)
 	if (!fz_drop_imp(ctx, font, &font->refs))
 		return;
 
-	free_resources(ctx, font);
 	if (font->t3lists)
 	{
+		free_resources(ctx, font);
 		for (i = 0; i < 256; i++)
-		{
 			if (font->t3lists[i])
 				fz_drop_display_list(ctx, font->t3lists[i]);
-		}
 		fz_free(ctx, font->t3procs);
 		fz_free(ctx, font->t3lists);
 		fz_free(ctx, font->t3widths);
@@ -143,10 +149,16 @@ fz_drop_font(fz_context *ctx, fz_font *font)
 		fz_drop_freetype(ctx);
 	}
 
-	fz_drop_buffer(ctx, font->ft_buffer);
-	fz_free(ctx, font->ft_filepath);
+	for (i = 0; i < 256; ++i)
+		fz_free(ctx, font->encoding_cache[i]);
+
+	fz_drop_buffer(ctx, font->buffer);
 	fz_free(ctx, font->bbox_table);
 	fz_free(ctx, font->width_table);
+	fz_free(ctx, font->advance_cache);
+	hb_lock(ctx);
+	hb_font_destroy(font->hb_font);
+	hb_unlock(ctx);
 	fz_free(ctx, font);
 }
 
@@ -175,12 +187,18 @@ fz_set_font_bbox(fz_context *ctx, fz_font *font, float xmin, float ymin, float x
  * Freetype hooks
  */
 
-struct fz_font_context_s {
+struct fz_font_context_s
+{
 	int ctx_refs;
 	FT_Library ftlib;
 	int ftlib_refs;
 	fz_load_system_font_func load_font;
 	fz_load_system_cjk_font_func load_cjk_font;
+
+	/* Cached fallback fonts */
+	struct { fz_font *serif, *sans; } fallback[256];
+	fz_font *symbol;
+	fz_font *emoji;
 };
 
 #undef __FTERRORS_H__
@@ -213,8 +231,19 @@ fz_keep_font_context(fz_context *ctx)
 
 void fz_drop_font_context(fz_context *ctx)
 {
+	int i;
+
 	if (!ctx)
 		return;
+
+	for (i = 0; i < nelem(ctx->font->fallback); ++i)
+	{
+		fz_drop_font(ctx, ctx->font->fallback[i].serif);
+		fz_drop_font(ctx, ctx->font->fallback[i].sans);
+	}
+	fz_drop_font(ctx, ctx->font->symbol);
+	fz_drop_font(ctx, ctx->font->emoji);
+
 	if (fz_drop_imp(ctx, ctx->font, &ctx->font->ctx_refs))
 		fz_free(ctx, ctx->font);
 }
@@ -263,6 +292,66 @@ fz_font *fz_load_system_cjk_font(fz_context *ctx, const char *name, int ros, int
 	return font;
 }
 
+fz_font *fz_load_fallback_font(fz_context *ctx, int script, int serif, int bold, int italic)
+{
+	const char *data;
+	int size;
+
+	if (script < 0 || script > nelem(ctx->font->fallback))
+		return NULL;
+
+	/* TODO: bold and italic */
+
+	if (serif)
+	{
+		if (ctx->font->fallback[script].serif)
+			return ctx->font->fallback[script].serif;
+		data = fz_lookup_noto_font(ctx, script, 1, &size);
+		if (data)
+		{
+			ctx->font->fallback[script].serif = fz_new_font_from_memory(ctx, NULL, data, size, 0, 0);
+			return ctx->font->fallback[script].serif;
+		}
+	}
+
+	if (ctx->font->fallback[script].sans)
+		return ctx->font->fallback[script].sans;
+	data = fz_lookup_noto_font(ctx, script, 0, &size);
+	if (data)
+	{
+		ctx->font->fallback[script].sans = fz_new_font_from_memory(ctx, NULL, data, size, 0, 0);
+		return ctx->font->fallback[script].sans;
+	}
+
+	return NULL;
+}
+
+fz_font *fz_load_fallback_symbol_font(fz_context *ctx)
+{
+	const char *data;
+	int size;
+	if (!ctx->font->symbol)
+	{
+		data = fz_lookup_noto_symbol_font(ctx, &size);
+		if (data)
+			ctx->font->symbol = fz_new_font_from_memory(ctx, NULL, data, size, 0, 0);
+	}
+	return ctx->font->symbol;
+}
+
+fz_font *fz_load_fallback_emoji_font(fz_context *ctx)
+{
+	const char *data;
+	int size;
+	if (!ctx->font->emoji)
+	{
+		data = fz_lookup_noto_emoji_font(ctx, &size);
+		if (data)
+			ctx->font->emoji = fz_new_font_from_memory(ctx, NULL, data, size, 0, 0);
+	}
+	return ctx->font->emoji;
+}
+
 static const struct ft_error ft_errors[] =
 {
 #include FT_ERRORS_H
@@ -333,76 +422,87 @@ fz_drop_freetype(fz_context *ctx)
 	fz_unlock(ctx, FZ_LOCK_FREETYPE);
 }
 
-fz_font *
-fz_new_font_from_file(fz_context *ctx, const char *name, const char *path, int index, int use_glyph_bbox)
-{
-	FT_Face face;
-	fz_font *font;
-	int fterr;
-
-	fz_keep_freetype(ctx);
-
-	fz_lock(ctx, FZ_LOCK_FREETYPE);
-	fterr = FT_New_Face(ctx->font->ftlib, path, index, &face);
-	fz_unlock(ctx, FZ_LOCK_FREETYPE);
-	if (fterr)
-	{
-		fz_drop_freetype(ctx);
-		fz_throw(ctx, FZ_ERROR_GENERIC, "freetype: cannot load font: %s", ft_error_string(fterr));
-	}
-
-	if (!name)
-		name = face->family_name;
-
-	font = fz_new_font(ctx, name, use_glyph_bbox, face->num_glyphs);
-	font->ft_face = face;
-	fz_set_font_bbox(ctx, font,
-		(float) face->bbox.xMin / face->units_per_EM,
-		(float) face->bbox.yMin / face->units_per_EM,
-		(float) face->bbox.xMax / face->units_per_EM,
-		(float) face->bbox.yMax / face->units_per_EM);
-	font->ft_filepath = fz_strdup(ctx, path);
-
-	return font;
-}
-
-fz_font *
-fz_new_font_from_memory(fz_context *ctx, const char *name, unsigned char *data, int len, int index, int use_glyph_bbox)
-{
-	FT_Face face;
-	fz_font *font;
-	int fterr;
-
-	fz_keep_freetype(ctx);
-
-	fz_lock(ctx, FZ_LOCK_FREETYPE);
-	fterr = FT_New_Memory_Face(ctx->font->ftlib, data, len, index, &face);
-	fz_unlock(ctx, FZ_LOCK_FREETYPE);
-	if (fterr)
-	{
-		fz_drop_freetype(ctx);
-		fz_throw(ctx, FZ_ERROR_GENERIC, "freetype: cannot load font: %s", ft_error_string(fterr));
-	}
-
-	if (!name)
-		name = face->family_name;
-
-	font = fz_new_font(ctx, name, use_glyph_bbox, face->num_glyphs);
-	font->ft_face = face;
-	fz_set_font_bbox(ctx, font,
-		(float) face->bbox.xMin / face->units_per_EM,
-		(float) face->bbox.yMin / face->units_per_EM,
-		(float) face->bbox.xMax / face->units_per_EM,
-		(float) face->bbox.yMax / face->units_per_EM);
-
-	return font;
-}
-
 fz_font *
 fz_new_font_from_buffer(fz_context *ctx, const char *name, fz_buffer *buffer, int index, int use_glyph_bbox)
 {
-	fz_font *font = fz_new_font_from_memory(ctx, name, buffer->data, buffer->len, index, use_glyph_bbox);
-	font->ft_buffer = fz_keep_buffer(ctx, buffer); /* remember buffer so we can drop it when we free the font */
+	FT_Face face;
+	TT_OS2 *os2;
+	fz_font *font;
+	int fterr;
+	FT_ULong tag, size, i, n;
+
+	fz_keep_freetype(ctx);
+
+	fz_lock(ctx, FZ_LOCK_FREETYPE);
+	fterr = FT_New_Memory_Face(ctx->font->ftlib, buffer->data, buffer->len, index, &face);
+	fz_unlock(ctx, FZ_LOCK_FREETYPE);
+	if (fterr)
+	{
+		fz_drop_freetype(ctx);
+		fz_throw(ctx, FZ_ERROR_GENERIC, "freetype: cannot load font: %s", ft_error_string(fterr));
+	}
+
+	if (!name)
+		name = face->family_name;
+
+	font = fz_new_font(ctx, name, use_glyph_bbox, face->num_glyphs);
+	font->ft_face = face;
+	fz_set_font_bbox(ctx, font,
+		(float) face->bbox.xMin / face->units_per_EM,
+		(float) face->bbox.yMin / face->units_per_EM,
+		(float) face->bbox.xMax / face->units_per_EM,
+		(float) face->bbox.yMax / face->units_per_EM);
+
+	font->is_mono = !!(face->face_flags & FT_FACE_FLAG_FIXED_WIDTH);
+	font->is_serif = 1;
+	font->is_bold = !!(face->style_flags & FT_STYLE_FLAG_BOLD);
+	font->is_italic = !!(face->style_flags & FT_STYLE_FLAG_ITALIC);
+
+	if (FT_IS_SFNT(face))
+	{
+		os2 = FT_Get_Sfnt_Table(face, FT_SFNT_OS2);
+		if (os2)
+			font->is_serif = !(os2->sFamilyClass & 2048); /* Class 8 is sans-serif */
+
+		FT_Sfnt_Table_Info(face, 0, NULL, &n);
+		for (i = 0; i < n; ++i)
+		{
+			FT_Sfnt_Table_Info(face, i, &tag, &size);
+			if (tag == TTAG_GDEF || tag == TTAG_GPOS || tag == TTAG_GSUB)
+				font->has_opentype = 1;
+		}
+	}
+
+	font->buffer = fz_keep_buffer(ctx, buffer);
+
+	return font;
+}
+
+fz_font *
+fz_new_font_from_memory(fz_context *ctx, const char *name, const char *data, int len, int index, int use_glyph_bbox)
+{
+	fz_buffer *buffer = fz_new_buffer_from_shared_data(ctx, data, len);
+	fz_font *font;
+	fz_try(ctx)
+		font = fz_new_font_from_buffer(ctx, name, buffer, index, use_glyph_bbox);
+	fz_always(ctx)
+		fz_drop_buffer(ctx, buffer);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+	return font;
+}
+
+fz_font *
+fz_new_font_from_file(fz_context *ctx, const char *name, const char *path, int index, int use_glyph_bbox)
+{
+	fz_buffer *buffer = fz_read_file(ctx, path);
+	fz_font *font;
+	fz_try(ctx)
+		font = fz_new_font_from_buffer(ctx, name, buffer, index, use_glyph_bbox);
+	fz_always(ctx)
+		fz_drop_buffer(ctx, buffer);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
 	return font;
 }
 
@@ -466,7 +566,7 @@ do_ft_render_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm
 
 	fz_adjust_ft_glyph_width(ctx, font, gid, &local_trm);
 
-	if (font->ft_italic)
+	if (font->fake_italic)
 		fz_pre_shear(&local_trm, SHEAR, 0);
 
 	/*
@@ -511,7 +611,7 @@ do_ft_render_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm
 			goto retry_unhinted;
 		}
 	}
-	else if (font->ft_hint)
+	else if (font->force_hinting)
 	{
 		/*
 		Enable hinting, but keep the huge char size so that
@@ -537,7 +637,7 @@ retry_unhinted:
 		}
 	}
 
-	if (font->ft_bold)
+	if (font->fake_bold)
 	{
 		FT_Outline_Embolden(&face->glyph->outline, strength * 64);
 		FT_Outline_Translate(&face->glyph->outline, -strength * 32, -strength * 32);
@@ -611,7 +711,7 @@ fz_render_ft_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm
 
 /* Takes the freetype lock, and returns with it held */
 static FT_Glyph
-do_render_ft_stroked_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm, const fz_matrix *ctm, fz_stroke_state *state)
+do_render_ft_stroked_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm, const fz_matrix *ctm, const fz_stroke_state *state)
 {
 	FT_Face face = font->ft_face;
 	float expansion = fz_matrix_expansion(ctm);
@@ -622,11 +722,12 @@ do_render_ft_stroked_glyph(fz_context *ctx, fz_font *font, int gid, const fz_mat
 	FT_Stroker stroker;
 	FT_Glyph glyph;
 	FT_Stroker_LineJoin line_join;
+	FT_Stroker_LineCap line_cap;
 	fz_matrix local_trm = *trm;
 
 	fz_adjust_ft_glyph_width(ctx, font, gid, &local_trm);
 
-	if (font->ft_italic)
+	if (font->fake_italic)
 		fz_pre_shear(&local_trm, SHEAR, 0);
 
 	m.xx = local_trm.a * 64; /* should be 65536 */
@@ -660,22 +761,19 @@ do_render_ft_stroked_glyph(fz_context *ctx, fz_font *font, int gid, const fz_mat
 		return NULL;
 	}
 
-#if FREETYPE_MAJOR * 10000 + FREETYPE_MINOR * 100 + FREETYPE_PATCH > 20405
-	/* New freetype */
 	line_join =
 		state->linejoin == FZ_LINEJOIN_MITER ? FT_STROKER_LINEJOIN_MITER_FIXED :
 		state->linejoin == FZ_LINEJOIN_ROUND ? FT_STROKER_LINEJOIN_ROUND :
 		state->linejoin == FZ_LINEJOIN_BEVEL ? FT_STROKER_LINEJOIN_BEVEL :
 		FT_STROKER_LINEJOIN_MITER_VARIABLE;
-#else
-	/* Old freetype */
-	line_join =
-		state->linejoin == FZ_LINEJOIN_MITER ? FT_STROKER_LINEJOIN_MITER :
-		state->linejoin == FZ_LINEJOIN_ROUND ? FT_STROKER_LINEJOIN_ROUND :
-		state->linejoin == FZ_LINEJOIN_BEVEL ? FT_STROKER_LINEJOIN_BEVEL :
-		FT_STROKER_LINEJOIN_MITER;
-#endif
-	FT_Stroker_Set(stroker, linewidth, (FT_Stroker_LineCap)state->start_cap, line_join, state->miterlimit * 65536);
+	line_cap =
+		state->start_cap == FZ_LINECAP_BUTT ? FT_STROKER_LINECAP_BUTT :
+		state->start_cap == FZ_LINECAP_ROUND ? FT_STROKER_LINECAP_ROUND :
+		state->start_cap == FZ_LINECAP_SQUARE ? FT_STROKER_LINECAP_SQUARE :
+		state->start_cap == FZ_LINECAP_TRIANGLE ? FT_STROKER_LINECAP_BUTT :
+		FT_STROKER_LINECAP_BUTT;
+
+	FT_Stroker_Set(stroker, linewidth, line_cap, line_join, state->miterlimit * 65536);
 
 	fterr = FT_Get_Glyph(face->glyph, &glyph);
 	if (fterr)
@@ -707,7 +805,7 @@ do_render_ft_stroked_glyph(fz_context *ctx, fz_font *font, int gid, const fz_mat
 }
 
 fz_pixmap *
-fz_render_ft_stroked_glyph_pixmap(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm, const fz_matrix *ctm, fz_stroke_state *state)
+fz_render_ft_stroked_glyph_pixmap(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm, const fz_matrix *ctm, const fz_stroke_state *state)
 {
 	FT_Glyph glyph = do_render_ft_stroked_glyph(ctx, font, gid, trm, ctm, state);
 	FT_BitmapGlyph bitmap = (FT_BitmapGlyph)glyph;
@@ -737,7 +835,7 @@ fz_render_ft_stroked_glyph_pixmap(fz_context *ctx, fz_font *font, int gid, const
 }
 
 fz_glyph *
-fz_render_ft_stroked_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm, const fz_matrix *ctm, fz_stroke_state *state)
+fz_render_ft_stroked_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm, const fz_matrix *ctm, const fz_stroke_state *state)
 {
 	FT_Glyph glyph = do_render_ft_stroked_glyph(ctx, font, gid, trm, ctm, state);
 	FT_BitmapGlyph bitmap = (FT_BitmapGlyph)glyph;
@@ -786,7 +884,7 @@ fz_bound_ft_glyph(fz_context *ctx, fz_font *font, int gid, fz_rect *bounds)
 
 	fz_adjust_ft_glyph_width(ctx, font, gid, &local_trm);
 
-	if (font->ft_italic)
+	if (font->fake_italic)
 		fz_pre_shear(&local_trm, SHEAR, 0);
 
 	m.xx = local_trm.a * 65536;
@@ -796,7 +894,7 @@ fz_bound_ft_glyph(fz_context *ctx, fz_font *font, int gid, fz_rect *bounds)
 	v.x = local_trm.e * 65536;
 	v.y = local_trm.f * 65536;
 
-	if (font->ft_hint)
+	if (font->force_hinting)
 	{
 		ft_flags = FT_LOAD_NO_BITMAP;
 	}
@@ -824,7 +922,7 @@ fz_bound_ft_glyph(fz_context *ctx, fz_font *font, int gid, fz_rect *bounds)
 		return bounds;
 	}
 
-	if (font->ft_bold)
+	if (font->fake_bold)
 	{
 		FT_Outline_Embolden(&face->glyph->outline, strength * scale);
 		FT_Outline_Translate(&face->glyph->outline, -strength * 0.5 * scale, -strength * 0.5 * scale);
@@ -926,12 +1024,12 @@ fz_outline_ft_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *tr
 
 	fz_adjust_ft_glyph_width(ctx, font, gid, &local_trm);
 
-	if (font->ft_italic)
+	if (font->fake_italic)
 		fz_pre_shear(&local_trm, SHEAR, 0);
 
 	fz_lock(ctx, FZ_LOCK_FREETYPE);
 
-	if (font->ft_hint)
+	if (font->force_hinting)
 	{
 		ft_flags = FT_LOAD_NO_BITMAP | FT_LOAD_IGNORE_TRANSFORM;
 		fterr = FT_Set_Char_Size(face, scale, scale, 72, 72);
@@ -951,7 +1049,7 @@ fz_outline_ft_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *tr
 		return NULL;
 	}
 
-	if (font->ft_bold)
+	if (font->fake_bold)
 	{
 		FT_Outline_Embolden(&face->glyph->outline, strength * scale);
 		FT_Outline_Translate(&face->glyph->outline, -strength * 0.5 * scale, -strength * 0.5 * scale);
@@ -1046,7 +1144,7 @@ fz_prepare_t3_glyph(fz_context *ctx, fz_font *font, int gid, int nested_depth)
 	if (dev->flags & FZ_DEVFLAG_BBOX_DEFINED)
 	{
 		assert(font->bbox_table != NULL);
-		assert(font->bbox_count > gid);
+		assert(font->glyph_count > gid);
 		font->bbox_table[gid] = dev->d1_rect;
 		fz_transform_rect(&font->bbox_table[gid], &font->t3matrix);
 	}
@@ -1218,38 +1316,36 @@ fz_render_t3_glyph_direct(fz_context *ctx, fz_device *dev, fz_font *font, int gi
 	font->t3run(ctx, font->t3doc, font->t3resources, contents, dev, &ctm, gstate, nested_depth);
 }
 
-#ifndef NDEBUG
 void
-fz_print_font(fz_context *ctx, FILE *out, fz_font *font)
+fz_print_font(fz_context *ctx, fz_output *out, fz_font *font)
 {
-	fprintf(out, "font '%s' {\n", font->name);
+	fz_printf(ctx, out, "font '%s' {\n", font->name);
 
 	if (font->ft_face)
 	{
-		fprintf(out, "\tfreetype face %p\n", font->ft_face);
+		fz_printf(ctx, out, "\tfreetype face %p\n", font->ft_face);
 		if (font->ft_substitute)
-			fprintf(out, "\tsubstitute font\n");
+			fz_printf(ctx, out, "\tsubstitute font\n");
 	}
 
 	if (font->t3procs)
 	{
-		fprintf(out, "\ttype3 matrix [%g %g %g %g]\n",
+		fz_printf(ctx, out, "\ttype3 matrix [%g %g %g %g]\n",
 			font->t3matrix.a, font->t3matrix.b,
 			font->t3matrix.c, font->t3matrix.d);
 
-		fprintf(out, "\ttype3 bbox [%g %g %g %g]\n",
+		fz_printf(ctx, out, "\ttype3 bbox [%g %g %g %g]\n",
 			font->bbox.x0, font->bbox.y0,
 			font->bbox.x1, font->bbox.y1);
 	}
 
-	fprintf(out, "}\n");
+	fz_printf(ctx, out, "}\n");
 }
-#endif
 
 fz_rect *
 fz_bound_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm, fz_rect *rect)
 {
-	if (font->bbox_table && gid < font->bbox_count)
+	if (font->bbox_table && gid < font->glyph_count)
 	{
 		if (fz_is_infinite_rect(&font->bbox_table[gid]))
 		{
@@ -1261,6 +1357,8 @@ fz_bound_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *trm, fz
 				font->bbox_table[gid] = fz_empty_rect;
 		}
 		*rect = font->bbox_table[gid];
+		if (fz_is_empty_rect(rect))
+			*rect = font->bbox;
 	}
 	else
 	{
@@ -1281,17 +1379,18 @@ fz_outline_glyph(fz_context *ctx, fz_font *font, int gid, const fz_matrix *ctm)
 
 int fz_glyph_cacheable(fz_context *ctx, fz_font *font, int gid)
 {
-	if (!font->t3procs || !font->t3flags || gid < 0 || gid >= font->bbox_count)
+	if (!font->t3procs || !font->t3flags || gid < 0 || gid >= font->glyph_count)
 		return 1;
 	return (font->t3flags[gid] & FZ_DEVFLAG_UNCACHEABLE) == 0;
 }
 
 static float
-fz_advance_ft_glyph(fz_context *ctx, fz_font *font, int gid)
+fz_advance_ft_glyph(fz_context *ctx, fz_font *font, int gid, int wmode)
 {
 	FT_Fixed adv;
 	int mask;
 
+	/* Substitute font widths. */
 	if (font->width_table)
 	{
 		if (gid < font->width_count)
@@ -1300,8 +1399,8 @@ fz_advance_ft_glyph(fz_context *ctx, fz_font *font, int gid)
 	}
 
 	mask = FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING | FT_LOAD_IGNORE_TRANSFORM;
-	/* if (font->wmode)
-		mask |= FT_LOAD_VERTICAL_LAYOUT; */
+	if (wmode)
+		mask |= FT_LOAD_VERTICAL_LAYOUT;
 	fz_lock(ctx, FZ_LOCK_FREETYPE);
 	FT_Get_Advance(font->ft_face, gid, mask, &adv);
 	fz_unlock(ctx, FZ_LOCK_FREETYPE);
@@ -1316,26 +1415,110 @@ fz_advance_t3_glyph(fz_context *ctx, fz_font *font, int gid)
 	return font->t3widths[gid];
 }
 
+void
+fz_get_glyph_name(fz_context *ctx, fz_font *font, int glyph, char *buf, int size)
+{
+	FT_Face face = font->ft_face;
+	if (face)
+	{
+		if (FT_HAS_GLYPH_NAMES(face))
+			FT_Get_Glyph_Name(face, glyph, buf, size);
+		else
+			fz_snprintf(buf, size, "%d", glyph);
+	}
+	else
+	{
+		fz_snprintf(buf, size, "%d", glyph);
+	}
+}
+
 float
-fz_advance_glyph(fz_context *ctx, fz_font *font, int gid)
+fz_advance_glyph(fz_context *ctx, fz_font *font, int gid, int wmode)
 {
 	if (font->ft_face)
-		return fz_advance_ft_glyph(ctx, font, gid);
+	{
+		if (wmode)
+			return fz_advance_ft_glyph(ctx, font, gid, 1);
+		if (gid >= 0 && gid < font->glyph_count && gid < MAX_ADVANCE_CACHE)
+		{
+			if (!font->advance_cache)
+			{
+				int i;
+				font->advance_cache = fz_malloc_array(ctx, font->glyph_count, sizeof(float));
+				for (i = 0; i < font->glyph_count; ++i)
+					font->advance_cache[i] = fz_advance_ft_glyph(ctx, font, i, 0);
+			}
+			return font->advance_cache[gid];
+		}
+
+		return fz_advance_ft_glyph(ctx, font, gid, 0);
+	}
 	if (font->t3procs)
 		return fz_advance_t3_glyph(ctx, font, gid);
 	return 0;
 }
 
-static int
-fz_encode_ft_character(fz_context *ctx, fz_font *font, int ucs)
-{
-	return FT_Get_Char_Index(font->ft_face, ucs);
-}
-
 int
 fz_encode_character(fz_context *ctx, fz_font *font, int ucs)
 {
 	if (font->ft_face)
-		return fz_encode_ft_character(ctx, font, ucs);
+	{
+		if (ucs >= 0 && ucs < 0x10000)
+		{
+			int pg = ucs >> 8;
+			int ix = ucs & 0xFF;
+			if (!font->encoding_cache[pg])
+			{
+				int i;
+				font->encoding_cache[pg] = fz_malloc_array(ctx, 256, sizeof(uint16_t));
+				for (i = 0; i < 256; ++i)
+					font->encoding_cache[pg][i] = FT_Get_Char_Index(font->ft_face, (pg << 8) + i);
+			}
+			return font->encoding_cache[pg][ix];
+		}
+		return FT_Get_Char_Index(font->ft_face, ucs);
+	}
 	return ucs;
 }
+
+/* FIXME: This should take language too eventually, to allow for fonts where we can select different
+ * languages using opentype features. */
+int
+fz_encode_character_with_fallback(fz_context *ctx, fz_font *user_font, int unicode, int script, fz_font **out_font)
+{
+	fz_font *font;
+	int gid;
+
+	gid = fz_encode_character(ctx, user_font, unicode);
+	if (gid > 0)
+		return *out_font = user_font, gid;
+
+	if (script == 0)
+		script = ucdn_get_script(unicode);
+
+	font = fz_load_fallback_font(ctx, script, user_font->is_serif, user_font->is_bold, user_font->is_italic);
+	if (font)
+	{
+		gid = fz_encode_character(ctx, font, unicode);
+		if (gid > 0)
+			return *out_font = font, gid;
+	}
+
+	font = fz_load_fallback_symbol_font(ctx);
+	if (font)
+	{
+		gid = fz_encode_character(ctx, font, unicode);
+		if (gid > 0)
+			return *out_font = font, gid;
+	}
+
+	font = fz_load_fallback_emoji_font(ctx);
+	if (font)
+	{
+		gid = fz_encode_character(ctx, font, unicode);
+		if (gid > 0)
+			return *out_font = font, gid;
+	}
+
+	return *out_font = user_font, 0;
+}
