@@ -146,7 +146,7 @@ static void jsB_new_RegExp(js_State *J)
 
 static void jsB_RegExp(js_State *J)
 {
-	if (js_gettop(J) == 2 && js_isregexp(J, 1))
+	if (js_isregexp(J, 1))
 		return;
 	jsB_new_RegExp(J);
 }
